# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: search/grpc/search.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x18search/grpc/search.proto\x12\x0e\x64\x61rsman.search\x1a\x1cgoogle/protobuf/struct.proto\"{\n\x1b\x43reateIndexedContentRequest\x12\x10\n\x08keywords\x18\x01 \x03(\t\x12\r\n\x05title\x18\x02 \x01(\t\x12\x0f\n\x07\x63ontent\x18\x03 \x01(\t\x12\x17\n\x0fitem_identifier\x18\x04 \x01(\t\x12\x11\n\titem_type\x18\x05 \x01(\t\"\x83\x01\n\x1c\x43reateIndexedContentResponse\x12\x14\n\x07message\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x11\n\x04\x63ode\x18\x02 \x01(\tH\x01\x88\x01\x01\x12%\n\x04\x64\x61ta\x18\x03 \x03(\x0b\x32\x17.google.protobuf.StructB\n\n\x08_messageB\x07\n\x05_code\"H\n\x1bSearchIndexedContentRequest\x12\x15\n\rsearch_phrase\x18\x01 \x01(\t\x12\x12\n\nitem_types\x18\x02 \x03(\t\"\x83\x01\n\x1cSearchIndexedContentResponse\x12\x14\n\x07message\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x11\n\x04\x63ode\x18\x02 \x01(\tH\x01\x88\x01\x01\x12%\n\x04\x64\x61ta\x18\x03 \x03(\x0b\x32\x17.google.protobuf.StructB\n\n\x08_messageB\x07\n\x05_code2\x87\x01\n\x1e\x43reateIndexedContentController\x12\x65\n\x06\x43reate\x12+.darsman.search.CreateIndexedContentRequest\x1a,.darsman.search.CreateIndexedContentResponse\"\x00\x32\x87\x01\n\x1eSearchIndexedContentController\x12\x65\n\x06Search\x12+.darsman.search.SearchIndexedContentRequest\x1a,.darsman.search.SearchIndexedContentResponse\"\x00\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'search.grpc.search_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  _globals['_CREATEINDEXEDCONTENTREQUEST']._serialized_start=74
  _globals['_CREATEINDEXEDCONTENTREQUEST']._serialized_end=197
  _globals['_CREATEINDEXEDCONTENTRESPONSE']._serialized_start=200
  _globals['_CREATEINDEXEDCONTENTRESPONSE']._serialized_end=331
  _globals['_SEARCHINDEXEDCONTENTREQUEST']._serialized_start=333
  _globals['_SEARCHINDEXEDCONTENTREQUEST']._serialized_end=405
  _globals['_SEARCHINDEXEDCONTENTRESPONSE']._serialized_start=408
  _globals['_SEARCHINDEXEDCONTENTRESPONSE']._serialized_end=539
  _globals['_CREATEINDEXEDCONTENTCONTROLLER']._serialized_start=542
  _globals['_CREATEINDEXEDCONTENTCONTROLLER']._serialized_end=677
  _globals['_SEARCHINDEXEDCONTENTCONTROLLER']._serialized_start=680
  _globals['_SEARCHINDEXEDCONTENTCONTROLLER']._serialized_end=815
# @@protoc_insertion_point(module_scope)
